/**
 * PithHelmet
 * Copyright 2003, Mike Solomon <mas63@cornell.edu>
 * $Header: /Users/msolomon/Developer/cvs/PithHelmet/PithHelmet.h,v 1.30 2003/11/13 17:29:04 msolomon Exp $
 */

#import <Foundation/Foundation.h>

// these get saved in com.apple.Safari - so use a prefix just in case there might
// be a naming collision
#define PH_PREF_KEY_ALLOWED_URL_COUNT @"PH_AllowedUrlCount"
#define PH_PREF_KEY_BLOCK_CROSS_SITE_CONTENT @"PH_BlockCrossSiteContent"
#define PH_PREF_KEY_BLOCKED_URL_COUNT @"PH_BlockedUrlCount"
#define PH_PREF_KEY_COLLAPSE_BLOCKED_CONTENT @"PH_CollapseBlockedContent"
#define PH_PREF_KEY_COLLAPSE_BLOCKED_LINKS @"PH_CollapseBlockedLinks"
#define PH_PREF_KEY_COLLAPSE_AD_SIZED_IMAGES @"PH_CollapseAdSizedImages"
#define PH_PREF_KEY_ENABLE_CONTEXTUAL_MENU @"PH_EnableContextualMenu"
#define PH_PREF_KEY_ENABLE_FILTERING @"PH_EnableFiltering"
#define PH_PREF_KEY_ENABLE_LOGGING @"PH_EnableLogging"
#define PH_PREF_KEY_ENABLE_TRANSMOGRIFIER @"PH_EnableTransmogrifier"
#define PH_PREF_KEY_PLAY_ANIMATED_GIFS @"PH_PlayAnimatedGIFs"
#define PH_PREF_KEY_RULES_OUTLINE_VIEW_FONT_SIZE @"PH_RulesOutlineViewFontSize"
#define PH_PREF_KEY_TOTAL_URL_COUNT @"PH_TotalUrlCount"

#define PH_PREF_VAL_PLAY_ANIMATED_GIFS_ALWAYS @"Always"
#define PH_PREF_VAL_PLAY_ANIMATED_GIFS_ONCE @"Once"
#define PH_PREF_VAL_PLAY_ANIMATED_GIFS_NEVER @"Never"

#define PH_PLIST_KEY_ALLOW @"Allow"
#define PH_PLIST_KEY_BLOCK @"Block"
#define PH_PLIST_KEY_ALLOW_DOMAINS @"Allow Domains"
#define PH_PLIST_KEY_BLOCK_DOMAINS @"Block Domains"
#define PH_PLIST_KEY_EXEMPT_SITES @"Exempt Sites"
#define PH_PLIST_KEY_TRANSMOGRIFICATIONS @"Transmogrifications"
#define PH_PLIST_KEY_VERSION @"Version"

// a little bonus for the kids
#define PH_PREF_KEY_ENABLE_DEBUG_MENU @"IncludeDebugMenu"

#define PH_ACTION_BLOCK @"block"
#define PH_ACTION_ALLOW @"allow"
#define PH_ACTION_TRANSMOGRIFY @"transmogrify"

#define PH_RULE_CROSS_SITE @"cross_site"

#define PH_PLIST_PATH @"~/Library/Application Support/PithHelmet/PithHelmet.plist"
#define PH_LOG_PATH @"~/Library/Application Support/PithHelmet/PithHelmet.log"

#define PH_STRING_TABLE @"PithHelmet"
#define PH_IDENTIFIER @"net.culater.PithHelmet"

#define PH_INFO_KEY_MIN_SAFARI_BUNDLE_VERSION @"MinSafariBundleVersion"
#define PH_INFO_KEY_MAX_SAFARI_BUNDLE_VERSION @"MaxSafariBundleVersion"


@interface PithHelmet : NSObject
{
	NSMutableArray* blockRules;
	NSMutableArray* allowRules;
	NSMutableArray* transmogrificationRules;

	NSMutableArray* exemptSiteRules;

	NSMutableString* collapseContentScriptlet;
	NSMutableDictionary* reloadUrlsWithoutFilter;
	
	int totalUrlCount;
	int blockedUrlCount;
	int allowedUrlCount;
		
	NSFileHandle* logFileHandle;
	
	BOOL loaded;
}

+ (void) load;
+ (PithHelmet*) sharedInstance;

- (NSString*) logAction:(NSString*)_action forUrl:(NSString*)_url withReferrer:(NSString*)_referrer byRule:(NSString*)_rule withResult:(NSString*)_result testMode:(BOOL)_testMode;
- (void) loadLogFile;
- (void) truncateLogFile;

- (void) checkFiles;
- (NSDictionary*) mergeFactoryPlist:(NSDictionary*)_factoryPlist intoUserPlist:(NSDictionary*)_userPlist;
- (void) makeBackupFile:(NSString*)_filePath;

- (id) readRulesFromFile:(NSString*)_filePath asMutable:(BOOL)_mutable;
- (void) loadRulesFromDictionary:(NSDictionary*)_rulesPlist;
- (void) saveRulesToFile:(NSString*)_filePath;
- (void) saveRulesDictionary:(NSDictionary*)_rulesPlist toFile:(NSString*)_filePath;

- (BOOL) allowResourceLoad:(NSURL*)_url withReferrer:(NSURL*)_referrer;
- (BOOL) allowResourceLoad:(NSURL*)_url withReferrer:(NSURL*)_referrer testMode:(BOOL)_testMode logResults:(NSString**)_logResults;
- (NSURL*) blockedFileForUrl:(NSURL*)_url;
- (NSString*) transmogrify:(NSURL*)_url;
- (NSString*) transmogrify:(NSURL*)_url testMode:(BOOL)_testMode logResults:(NSString**)_logResults;

- (void) toggleContentFiltering;
- (void) reloadRules;
- (void) revertToFactoryRules;
- (void) reloadWithoutFilter;
- (void) startLoadWithoutFilter:(NSURL*)_url;
- (BOOL) shouldLoadWithoutFilter:(NSURL*)_url;
- (void) finishLoad:(NSURL*)_url;

- (void) readUserDefaults;
- (void) saveUserDefaults;

- (int) totalUrlCount;
- (int) allowedUrlCount;
- (int) blockedUrlCount;
- (float) blockedPercentage;

- (BOOL) shouldCollapseContentForUrl:(NSURL*)_url;

- (void) setLoaded:(BOOL)_loaded;
- (BOOL) isLoaded;
- (NSString*) version;
- (NSString*) nibName;
- (NSString*) bundleVersion;

- (NSString*) collapseContentScriptlet;
- (void) buildContentScriptlet;

- (void) setBlockRules:(NSMutableArray*)_blockRules;
- (void) setAllowRules:(NSMutableArray*)_allowRules;
- (void) setExemptSiteRules:(NSMutableArray*)_rules;
- (NSArray*) blockRules;
- (NSArray*) allowRules;
- (NSArray*) exemptSiteRules;

+ (void) errorAlert:(NSString*)_message withDetails:(NSString*)_details, ...;

@end
