/**
 * PithHelmet
 * @author Mike Solomon
 * Copyright 2003 Mike Solomon. All rights reserved.
 * $Header: /Users/msolomon/Developer/cvs/PithHelmet/PithHelmetBrowserWebView.m,v 1.5 2003/08/22 06:39:48 msolomon Exp $
 */

RCS_ID("$Id: PithHelmetBrowserWebView.m,v 1.5 2003/08/22 06:39:48 msolomon Exp $")

#import "PithHelmet.h"
#import "PithHelmetBrowserWebView.h"
#import "PithHelmetPreferences.h"


@implementation PithHelmetBrowserWebView

- webView:(id)_webView contextMenuItemsForElement:(NSDictionary*)_element defaultMenuItems:(id)_defaultItems
{
	DLog1(DLOG_40, (@"element %@", _element));
	DLog1(DLOG_0, (@"defaultMenuItems %@", _defaultItems));

	NSArray* contextMenuItems = [super webView:_webView contextMenuItemsForElement:_element defaultMenuItems:_defaultItems];
	
	if ([[NSUserDefaults standardUserDefaults] boolForKey:PH_PREF_KEY_ENABLE_CONTEXTUAL_MENU] == YES)
	{
		NSMutableArray* myContextMenuItems = [NSMutableArray arrayWithArray:contextMenuItems];
		
		if ([_element objectForKey:WebElementImageURLKey] != nil)
		{
			static NSArray* imageContextMenuItems = nil;
			static NSString* blockImagesTitleFormat = nil;
			// fixme: this code is fragile - it relies on the fact that the nib only has one menu item to add right now
			if (imageContextMenuItems == nil)
			{
				NSMenu* nibContextMenu = [[PithHelmetPreferences sharedInstance] imageContextMenuAdditions];
				imageContextMenuItems = [[NSMutableArray arrayWithArray:[nibContextMenu itemArray]] retain];
				NSEnumerator* menuItemsEnumerator = [imageContextMenuItems objectEnumerator];
				NSMenuItem* menuItem;
				while (menuItem = [menuItemsEnumerator nextObject])
				{
					blockImagesTitleFormat = [[menuItem title] copy];
					[nibContextMenu removeItem:menuItem];
				}
			}

			NSEnumerator* menuItemsEnumerator = [imageContextMenuItems objectEnumerator];
			NSMenuItem* menuItem;
			while (menuItem = [menuItemsEnumerator nextObject])
			{
				// note: this will get unset when the event is processed in PithHelmetPreferences
				// fixme: is this good design?
				NSURL* imageURL = [_element objectForKey:WebElementImageURLKey];
				[menuItem setTitle:[NSString stringWithFormat:blockImagesTitleFormat, [imageURL host]]];
				[menuItem setRepresentedObject:imageURL];
			}
			[myContextMenuItems addObjectsFromArray:imageContextMenuItems];
	
			DLog1(DLOG_20, (@"myContextMenuItems <%@> %@", [myContextMenuItems class], myContextMenuItems));
			return myContextMenuItems;
		}
		else
		{
			static NSArray* webContextMenuItems = nil;
			static NSString* neverBlockTitleFormat = nil;
			// fixme: this code is fragile - it relies on the fact that the nib only has one menu item to add right now
			if (webContextMenuItems == nil)
			{
				NSMenu* nibContextMenu = [[PithHelmetPreferences sharedInstance] webContextMenuAdditions];
				webContextMenuItems = [[NSMutableArray arrayWithArray:[nibContextMenu itemArray]] retain];
				NSEnumerator* menuItemsEnumerator = [webContextMenuItems objectEnumerator];
				NSMenuItem* menuItem;
				while (menuItem = [menuItemsEnumerator nextObject])
				{
					neverBlockTitleFormat = [[menuItem title] copy];
					[nibContextMenu removeItem:menuItem];
				}
			}

			NSEnumerator* menuItemsEnumerator = [webContextMenuItems objectEnumerator];
			NSMenuItem* menuItem;
			while (menuItem = [menuItemsEnumerator nextObject])
			{
				// note: this will get unset when the event is processed in PithHelmetPreferences
				// fixme: is this good design?
				NSURL* pageURL = [[[(WebFrame*)[_element objectForKey:WebElementFrameKey] dataSource] request] URL];
				[menuItem setTitle:[NSString stringWithFormat:neverBlockTitleFormat, [pageURL host]]];
				[menuItem setRepresentedObject:pageURL];
			}
			[myContextMenuItems addObjectsFromArray:webContextMenuItems];
	
			DLog1(DLOG_20, (@"myContextMenuItems <%@> %@", [myContextMenuItems class], myContextMenuItems));
			return myContextMenuItems;
		}
	}
	DLog1(DLOG_20, (@"contextMenuItems <%@> %@", [contextMenuItems class], contextMenuItems));
	return contextMenuItems;
}

- (void)decidePolicyForAction:(NSDictionary*) actionInformation request:(NSURLRequest*) request frame:(WebFrame*) frame newFrameName:(id)newFrameName decisionListener:(id<WebPolicyDecisionListener>) listener
{
	DLog1(DLOG_20, (@"decidePolicyForAction: %@", actionInformation));
	[super decidePolicyForAction:actionInformation request:request frame:frame newFrameName:newFrameName decisionListener:listener];
}

- (void) webView: (WebView *) sender decidePolicyForMIMEType: (NSString *) type request: (NSURLRequest *) request frame: (WebFrame *)frame decisionListener: (id<WebPolicyDecisionListener>) listener 
{
	DLog1(DLOG_20, (@"decidePolicyForMIME: %@", type));
	[super webView:sender decidePolicyForMIMEType:type request:request frame:frame decisionListener: listener]; 
}

- (void) webView: (WebView *) sender decidePolicyForNavigationAction: (NSDictionary *) actionInformation request: (NSURLRequest *) request frame: (WebFrame *) frame decisionListener: (id<WebPolicyDecisionListener>) listener 
{
	DLog1(DLOG_20, (@"decidePolicyForNavigationAction: %@", actionInformation));
	DLog1(DLOG_20, (@"decidePolicyForNavigationAction: %@|%@|%@|%@", sender, request, frame, listener));
	[super webView:sender decidePolicyForNavigationAction:actionInformation request:request frame:frame decisionListener:listener];
}

- (void) webView: (WebView *) sender decidePolicyForNewWindowAction: (NSDictionary *) actionInformation request: (NSURLRequest *) request newFrameName: (NSString *) frameName decisionListener: (id<WebPolicyDecisionListener>) listener 
{
	DLog1(DLOG_20, (@"decidePolicyForNewWindowAction: %@", actionInformation));
	[super webView:sender decidePolicyForNewWindowAction:actionInformation request:request newFrameName:frameName decisionListener:listener];
}

@end
