/**
 * PithHelmet
 * Copyright 2003, Mike Solomon <mas63@cornell.edu>
 * $Header: /Users/msolomon/Developer/cvs/PithHelmet/PithHelmetPreferences.h,v 1.20 2003/08/17 08:49:48 msolomon Exp $
 */

#import <Cocoa/Cocoa.h>
#import "NSPreferences.h"
#import "PithHelmetRuleNode.h"

#define PH_IMAGE_NAME @"PithHelmet"
#define COLUMNID_NAME @"NameColumn"

#define PH_MENU_TAG_BLOCK_DOMAIN       1
#define PH_MENU_TAG_BLOCK_URL          2
#define PH_MENU_TAG_ALLOW_DOMAIN       3
#define PH_MENU_TAG_ALLOW_URL          4
#define PH_MENU_TAG_TRANSMOGRIFICATION 5

#define PH_MENU_TAG_REGEX_URL_MATCH    6
#define PH_MENU_TAG_REGEX_DOMAIN_MATCH 7
#define PH_MENU_TAG_DOMAIN_MATCH       8

#define PH_MENU_TAG_REMOVE_RULE        9

#define PH_MENU_TAG_GIF_ANIMATIONS_PLAY_ALWAYS        10
#define PH_MENU_TAG_GIF_ANIMATIONS_PLAY_ONCE          11
#define PH_MENU_TAG_GIF_ANIMATIONS_PLAY_NEVER         12


@interface PithHelmetPreferences : NSPreferencesModule
{
	IBOutlet NSTextField *buildLabel;
	IBOutlet NSButton *collapseBlockedContentCheckbox;
	IBOutlet NSButton *collapseAdSizedImagesCheckbox;
	IBOutlet NSButton *collapseBlockedLinksCheckbox;
	IBOutlet NSButton *blockCrossSiteContentCheckbox;
	IBOutlet NSButton *enableContextualMenuCheckbox;
	IBOutlet NSButton *enableLoggingCheckbox;
	IBOutlet NSButton *enableTransmogrifierCheckbox;
	IBOutlet NSButton *enableDebugMenuCheckbox;
	IBOutlet NSOutlineView *rulesOutlineView;
	IBOutlet NSTextField *statsLabel;
	IBOutlet NSTextView *testResultTextView;
	IBOutlet NSTextField *testUrlTextView;
	IBOutlet NSMenu *safariMenuAdditions;
	IBOutlet NSMenu *viewMenuAdditions;
	IBOutlet NSMenu *contextMenu;
	IBOutlet NSMenu *imageContextMenuAdditions;
	IBOutlet NSMenu *webContextMenuAdditions;
	IBOutlet NSMenuItem *addNewRuleMenuItem;
	IBOutlet NSMenuItem *setRuleTypeMenuItem;
	IBOutlet NSMenuItem *removeRuleMenuItem;
	IBOutlet NSPopUpButton *playAnimatedGIFsPopUp;
 
	
	// holds the formatting string for statsLabel - grabbed from the initial string specified in the nib
	NSString* statsLabelFormat;
	PithHelmetRuleNode* treeData;
	PithHelmetRuleNode* blockRulesGroup;
	PithHelmetRuleNode* allowRulesGroup;
	PithHelmetRuleNode* exemptSiteRulesGroup;
}

- (IBAction)aboutPithHelmet:(id)sender;
- (IBAction)addRule:(id)sender;
- (IBAction)blockImagesFromServer:(id)sender;
- (IBAction)clearRulesTest:(id)sender;
- (IBAction)donate:(id)sender;
- (IBAction)help:(id)sender;
- (IBAction)neverBlockContentFromServer:(id)sender;
- (IBAction)performRulesTest:(id)sender;
- (IBAction)regularExpressionsHelp:(id)sender;
- (IBAction)reloadPageUnfiltered:(id)sender;
- (IBAction)reloadRules:(id)sender;
- (IBAction)removeRule:(id)sender;
- (IBAction)reportBug:(id)sender;
- (IBAction)revertToFactoryRules:(id)sender;
- (IBAction)saveRules:(id)sender;
- (IBAction)setRuleType:(id)sender;
- (IBAction)setPlayAnimatedGIFs:(id)sender;
- (IBAction)toggleCheckbox:(id)sender;
- (IBAction)toggleContentFiltering:(id)sender;
- (IBAction)truncateLog:(id)sender;
 
+ (BOOL) validateRule:(PithHelmetRuleNodeData*)_ruleData errorIndex:(int*)_errorIndex errorMessage:(NSString**)_errorMessage;
+ (NSMutableArray*) makeRulesArrayFromRuleNode:(PithHelmetRuleNode*)_ruleGroupNode;
+ (NSImage*) preloadImage:(NSString*)_name;
- (void) truncateSheetDidEnd:(NSWindow *)_sheet returnCode:(int)_returnCode contextInfo:(void *)_contextInfo;
- (void) errorAlert:(NSString*)_message withDetails:(NSString*)_details, ...;
- (NSMenu*) contextMenu;
- (NSMenu*) imageContextMenuAdditions;
- (NSMenu*) webContextMenuAdditions;


@end
