/**
 * PithHelmet
 * Copyright 2003, Mike Solomon <mas63@cornell.edu>
 * $Header: /Users/msolomon/Developer/cvs/PithHelmet/PithHelmetPreferences.m,v 1.41 2003/11/13 17:29:04 msolomon Exp $
 */

RCS_ID("$Id: PithHelmetPreferences.m,v 1.41 2003/11/13 17:29:04 msolomon Exp $")

#import "PithHelmet.h"
#import "PithHelmetPreferences.h"
#import "PithHelmetRuleNode.h"

@implementation PithHelmetPreferences

- (void) readRulesFromPithHelmet
{
	// load up the outline data wish i could stick this in PithHelmet class...
	// not sure how to best separate out the data	
	
	NSEnumerator* rules;
	PithHelmetRuleNodeData *ruleData;

	rules = [[[PithHelmet sharedInstance] blockRules] objectEnumerator];
	[blockRulesGroup removeChildren];
	while (ruleData = [rules nextObject])
		[blockRulesGroup appendChild:[PithHelmetRuleNode treeNodeWithData:ruleData]];
	
	rules = [[[PithHelmet sharedInstance] allowRules] objectEnumerator];
	[allowRulesGroup removeChildren];
	while (ruleData = [rules nextObject])
		[allowRulesGroup appendChild:[PithHelmetRuleNode treeNodeWithData:ruleData]];

	rules = [[[PithHelmet sharedInstance] exemptSiteRules] objectEnumerator];
	[exemptSiteRulesGroup removeChildren];
	while (ruleData = [rules nextObject])
		[exemptSiteRulesGroup appendChild:[PithHelmetRuleNode treeNodeWithData:ruleData]];
}

- (IBAction)blockImagesFromServer:(id)sender
{
	DLog1(DLOG_30, (@"blockImagesFromServer <%@> %@", [sender class], sender));
	DLog1(DLOG_30, (@"representedObject <%@> %@", [[sender representedObject] class], [sender representedObject]));

	NSURL* imageUrl = [sender representedObject];
	if (imageUrl == nil)
		return;

	NS_DURING
		NSMutableString* domainString = [NSMutableString stringWithString:[imageUrl host]];
	// replace . with \. so they match correctly in the pattern
	[domainString replaceOccurrencesOfString:@"." withString:@"\\." options:NSLiteralSearch range:NSMakeRange(0, [domainString length])];

	NSString* ruleString = [NSString stringWithFormat:@"%@.*\\.(?:gif|jpg|png)", domainString];
	PithHelmetRuleNode* newItem = [PithHelmetRuleNode ruleNodeWithRule:ruleString andType:PH_RULE_TYPE_REGEX_MATCH];
	[(PithHelmetRuleNodeData*)[newItem nodeData] setSubresourceRule:YES];
	[blockRulesGroup appendChild:newItem];
	[self saveChanges];

	NSWindow* window = [[NSPreferences sharedPreferences] window];
	if (window != nil && [window isVisible] == YES)
		[rulesOutlineView reloadData];

	NS_HANDLER
		NSLog(@"%@: %@", [localException name], [localException reason]);
	[PithHelmet errorAlert:NSLocalizedStringFromTableInBundle(@"An error occurred while saving PithHelmet rules.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:[localException reason]];
	NS_ENDHANDLER
}

- (IBAction)neverBlockContentFromServer:(id)sender
{
	DLog1(DLOG_30, (@"neverBlockContentFromServer <%@> %@", [sender class], sender));
	DLog1(DLOG_30, (@"representedObject <%@> %@", [[sender representedObject] class], [sender representedObject]));

	NSURL* url = [sender representedObject];
	if (url == nil)
		return;

	NS_DURING
		NSMutableString* domainString = [NSMutableString stringWithString:[url host]];
	PithHelmetRuleNode* newItem = [PithHelmetRuleNode ruleNodeWithRule:domainString andType:PH_RULE_TYPE_DOMAIN_MATCH];
	[exemptSiteRulesGroup appendChild:newItem];
	[self saveChanges];

	NSWindow* window = [[NSPreferences sharedPreferences] window];
	if (window != nil && [window isVisible] == YES)
		[rulesOutlineView reloadData];

	NS_HANDLER
		NSLog(@"%@: %@", [localException name], [localException reason]);
	[PithHelmet errorAlert:NSLocalizedStringFromTableInBundle(@"An error occurred while saving PithHelmet rules.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:[localException reason]];
	NS_ENDHANDLER
}

- (IBAction)addRule:(id)sender
{
	DLog1(DLOG_20, (@"addRule: <%@> %@", [sender class], sender));

	PithHelmetRuleNode* groupItem;

	if ([sender isKindOfClass:[NSMenuItem class]] == YES)
	{
		NSMenuItem* menuItem = (NSMenuItem*)sender;
		
		switch ([menuItem tag])
		{
			case PH_MENU_TAG_BLOCK_DOMAIN:
				groupItem = blockRulesGroup;
				break;

			case PH_MENU_TAG_BLOCK_URL:
				groupItem = blockRulesGroup;
				break;

			case PH_MENU_TAG_ALLOW_DOMAIN:
				groupItem = allowRulesGroup;
				break;

			case PH_MENU_TAG_ALLOW_URL:
				groupItem = allowRulesGroup;
				break;

			case PH_MENU_TAG_TRANSMOGRIFICATION:
				// todo:
			default:
				return;
		}
	}
	else
	{
		int selectedRow = [rulesOutlineView selectedRow];
	
		// if selection is empty, how do we know where to add a rule?
		// let's just beep
		if (selectedRow == -1)
		{
			NSBeep();
			return;
		}
	
		PithHelmetRuleNode* item = (PithHelmetRuleNode*)[rulesOutlineView itemAtRow:selectedRow];
		// if this is a group node, this is what we want, else iterate to get the next group up
		// FIXME: we don't handle the recursion properly here
		groupItem = [item isGroup] ? item : (PithHelmetRuleNode*)[item nodeParent];
	}

	// need to decide rule type by which group we are adding to
	PithHelmetRuleNode* newItem = nil;
	if (groupItem == allowRulesGroup || groupItem == blockRulesGroup)
		newItem = [PithHelmetRuleNode ruleNodeWithRule:@"" andType:PH_RULE_TYPE_REGEX_MATCH];
	else if (groupItem == exemptSiteRulesGroup)
		newItem = [PithHelmetRuleNode ruleNodeWithRule:@"" andType:PH_RULE_TYPE_DOMAIN_MATCH];

	[groupItem appendChild:newItem];
	[rulesOutlineView reloadItem:groupItem reloadChildren:YES];
	
	if ([rulesOutlineView isItemExpanded:groupItem] == NO)
		[rulesOutlineView expandItem:groupItem];

	int selectedRow = [rulesOutlineView rowForItem:newItem];
	[rulesOutlineView selectRow:selectedRow byExtendingSelection:NO];
	[rulesOutlineView editColumn:0 row:selectedRow withEvent:nil select:YES];
}

- (BOOL) validateMenuItem: (id <NSMenuItem>) menuItem
{
	DLog1(DLOG_20, (@"menuItem: %@", menuItem));

	int selectedRow = [rulesOutlineView selectedRow];
	PithHelmetRuleNode* item = (selectedRow != -1) ? (PithHelmetRuleNode*)[rulesOutlineView itemAtRow:selectedRow] : nil;

	if ([menuItem tag] == PH_MENU_TAG_TRANSMOGRIFICATION)
		return NO;

	if ([menuItem tag] >= PH_MENU_TAG_BLOCK_DOMAIN && [menuItem tag] <= PH_MENU_TAG_REMOVE_RULE)
	{
		if (item == nil)
			return NO;
		
		return ![item isGroup];
	}
	
	return YES;
}

- (IBAction)setRuleType:(id)sender
{
	DLog1(DLOG_20, (@"setRuleType: <%@> %@", [sender class], sender));
	int selectedRow = [rulesOutlineView selectedRow];
	
	// if selection is empty, nothing to do,  let's just beep
	if (selectedRow == -1)
	{
		NSBeep();
		return;
	}
	
	PithHelmetRuleNode* item = (PithHelmetRuleNode*)[rulesOutlineView itemAtRow:selectedRow];
	// if this is a group node, just bail
	if ([item isGroup] == YES)
	{
		NSBeep();
		return;
	}

	NSMenuItem* menuItem = (NSMenuItem*)sender;
	NSString* ruleType = nil;
	switch ([menuItem tag])
	{
		case PH_MENU_TAG_REGEX_URL_MATCH:
			ruleType = PH_RULE_TYPE_REGEX_MATCH;
			break;

		case PH_MENU_TAG_REGEX_DOMAIN_MATCH:
			ruleType = PH_RULE_TYPE_REGEX_DOMAIN_MATCH;
			break;

		case PH_MENU_TAG_DOMAIN_MATCH:
			ruleType = PH_RULE_TYPE_DOMAIN_MATCH;
			break;

		default:
			NSLog(@"invalid rule tag: %d", [menuItem tag]); 
			return;
	}

	// todo: need to decide rule type by which group we are adding to
	PithHelmetRuleNodeData* oldRule = (PithHelmetRuleNodeData*)[item nodeData];
	PithHelmetRuleNodeData* newRule = [PithHelmetRuleNodeData ruleDataWithRule:[oldRule rule] andType:ruleType];
	if (newRule == nil)
	{
		[self errorAlert:NSLocalizedStringFromTableInBundle(@"An error occurred changing the rule type.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:NSLocalizedStringFromTableInBundle(@"Can't convert rule to type '%@'", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert details, substitute rule type string"), ruleType];
		return;
	}

	[item setNodeData:newRule];
	[rulesOutlineView reloadItem:item reloadChildren:NO];
}

- (IBAction)removeRule:(id)sender
{
	int selectedRow = [rulesOutlineView selectedRow];
	
	// if selection is empty, how do we know what rule to remove?
	// let's just beep
	if (selectedRow == -1)
	{
		NSBeep();
		return;
	}

	PithHelmetRuleNode* item = (PithHelmetRuleNode*)[rulesOutlineView itemAtRow:selectedRow];
	// if this is a group node, we want to skip this for now, you can only remove leaf nodes
	if ([item isGroup] == YES)
	{
		NSBeep();
		return;
	}
	
	[item removeFromParent];
	// FIXME: reloadData seems like overkill - we know what we just removed, but the line below causes
	// a crash
	// [rulesOutlineView reloadItem:[item nodeParent] reloadChildren:YES];
	[rulesOutlineView reloadData];
}

- (IBAction)saveRules:(id)sender
{
	[rulesOutlineView _endMyEditing];
	NS_DURING
		[self saveChanges];
	NS_HANDLER
		NSLog(@"%@: %@", [localException name], [localException reason]);
	[PithHelmet errorAlert:NSLocalizedStringFromTableInBundle(@"An error occurred while saving PithHelmet rules.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:[localException reason]];
	NS_ENDHANDLER
}

- (IBAction)reloadRules:(id)sender
{
	[rulesOutlineView _endMyEditing];

	NS_DURING
		[[PithHelmet sharedInstance] reloadRules];
	NS_HANDLER
		NSLog(@"%@: %@", [localException name], [localException reason]);
	[PithHelmet errorAlert:NSLocalizedStringFromTableInBundle(@"An error occurred while reloading PithHelmet rules.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:[localException reason]];
	NS_ENDHANDLER

	[self readRulesFromPithHelmet];
	[rulesOutlineView reloadData];
}


- (IBAction)aboutPithHelmet:(id)sender
{
	NSBundle* bundle = [NSBundle bundleWithIdentifier:PH_IDENTIFIER];
	NSString* aboutFile = [bundle pathForResource:@"PithHelmet" ofType:@"html"];
	[[NSWorkspace sharedWorkspace] openURL:[NSURL fileURLWithPath:aboutFile]];
}


- (IBAction)help:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://culater.net/wikir/wikir.cgi?mode=read&arg=PithHelmetFAQ"]];
}


- (IBAction)regularExpressionsHelp:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://culater.net/wikir/wikir.cgi?mode=read&arg=PithHelmetFAQ"]];
}


- (IBAction)donate:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"https://www.paypal.com/xclick/business=donations%40culater.net&item_name=PithHelmet+Development&no_shipping=1&no_note=1&tax=0&currency_code=USD"]];
}

- (IBAction)clearRulesTest:(id)sender
{
	[testUrlTextView setStringValue:@""];
	[testResultTextView setString:@""];
}

- (IBAction)truncateLog:(id)sender
{
	NS_DURING
		NSWindow* window = [[NSPreferences sharedPreferences] window];
	
	if (window != nil && [window isVisible] == YES)
		NSBeginAlertSheet(
			NSLocalizedStringFromTableInBundle(@"PithHelmet Warning", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Alert title"),
			NSLocalizedStringFromTableInBundle(@"Cancel", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Cancel button"),
			NSLocalizedStringFromTableInBundle(@"Truncate", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Truncate button"),
			nil,              // no third button
			window,           // window sheet is attached to
			self,             // we'll be our own delegate
			@selector(truncateSheetDidEnd:returnCode:contextInfo:),
			nil,              // no need for did-dismiss selector
			nil,              // context info
			NSLocalizedStringFromTableInBundle(@"Truncating the log will delete it permanently. Continue?", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Truncate log warning text"),
			nil);             // no parameters in message

	NS_HANDLER
		[self errorAlert:NSLocalizedStringFromTableInBundle(@"Error truncating PithHelmet log.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:[localException reason]];
	NS_ENDHANDLER
}

- (void) truncateSheetDidEnd:(NSWindow *)_sheet returnCode:(int)_returnCode contextInfo:(void *)_contextInfo
{
	if (_returnCode != NSAlertAlternateReturn)
		return;

	[[PithHelmet sharedInstance] truncateLogFile];
	[[PithHelmet sharedInstance] loadLogFile];
}


- (IBAction)revertToFactoryRules:(id)sender
{
	NS_DURING
		NSWindow* window = [[NSPreferences sharedPreferences] window];
	
	if (window != nil && [window isVisible] == YES)
		NSBeginAlertSheet(
			NSLocalizedStringFromTableInBundle(@"PithHelmet Warning", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Alert title"),
			NSLocalizedStringFromTableInBundle(@"Cancel", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Cancel button"),
			NSLocalizedStringFromTableInBundle(@"Revert", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Revert button"),
			nil,              // no third button
			window,           // window sheet is attached to
			self,             // we'll be our own delegate
			@selector(revertSheetDidEnd:returnCode:contextInfo:),
			nil,              // no need for did-dismiss selector
			nil,              // context info
			NSLocalizedStringFromTableInBundle(@"Reverting to factory rules will delete all custom rules permanently. Continue?", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Revert rules warning text"),
			nil);             // no parameters in message

	NS_HANDLER
		[self errorAlert:NSLocalizedStringFromTableInBundle(@"Error reverting to factory rules.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:[localException reason]];
	NS_ENDHANDLER
}

- (void) revertSheetDidEnd:(NSWindow *)_sheet returnCode:(int)_returnCode contextInfo:(void *)_contextInfo
{
	if (_returnCode != NSAlertAlternateReturn)
		return;

	[[PithHelmet sharedInstance] revertToFactoryRules];
	[self reloadRules:nil];
}


+ (NSMutableArray*) makeRulesArrayFromRuleNode:(PithHelmetRuleNode*)_ruleGroupNode
{
	NSArray* rulesNodes = [_ruleGroupNode children];
	NSMutableArray* rules = [NSMutableArray arrayWithCapacity:[rulesNodes count]];
	NSEnumerator* e = [rulesNodes objectEnumerator];
	PithHelmetRuleNode* node = nil;
	while (node = [e nextObject])
	{
		PithHelmetRuleNodeData* ruleData = (PithHelmetRuleNodeData*)[node nodeData];
		[rules addObject:ruleData];
	}
	return rules;
}

+ (BOOL) validateRule:(PithHelmetRuleNodeData*)_ruleData errorIndex:(int*)_errorIndex errorMessage:(NSString**)_errorMessage
{
	NSString* ruleType = [_ruleData type];
	if ([ruleType isEqualToString:PH_RULE_TYPE_REGEX_MATCH] == YES ||
			[ruleType isEqualToString:PH_RULE_TYPE_REGEX_DOMAIN_MATCH] == YES)
	{
		NSString* ruleString = [_ruleData rule];
		if ([ruleString length] == 0)
		{
			*_errorIndex = 0;
			*_errorMessage = NSLocalizedStringFromTableInBundle(@"empty string", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error message for rules validation");
			DLog1(DLOG_10, (@"length validation error at index %d : %@", *_errorIndex, *_errorMessage));
			return NO;
		}	
		
		int i;
		for (i = [ruleString length] - 1; i >= 0; i--)
		{
			// check for unescaped / characters - the so-called "leaning toothpick"
			// this is here because of the javascript syntax
			if ([ruleString characterAtIndex:i] == '/' && (!(i - 1 >= 0) || [ruleString characterAtIndex:i-1] != '\\'))
			{
				*_errorIndex = i;
				*_errorMessage = NSLocalizedStringFromTableInBundle(@"unescaped / character - replace with \\/", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error message for rules validation");
				DLog1(DLOG_10, (@"toothpick validation error at index %d : %@", *_errorIndex, *_errorMessage));
				return NO;
			}
		}
	
		return [PCRegExp validatePattern:ruleString errorIndex:_errorIndex errorMessage:_errorMessage];
	}
	else if ([ruleType isEqualToString:PH_RULE_TYPE_DOMAIN_MATCH])
	{
		NSString* ruleString = [_ruleData rule];
		if ([ruleString length] == 0)
		{
			*_errorIndex = 0;
			*_errorMessage = NSLocalizedStringFromTableInBundle(@"empty string", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error message for rules validation");
			DLog1(DLOG_10, (@"length validation error at index %d : %@", *_errorIndex, *_errorMessage));
			return NO;
		}
		
		return YES;
	}

	*_errorIndex = -1;
	*_errorMessage = [NSString stringWithFormat:NSLocalizedStringFromTableInBundle(@"unknown rule type: %@", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error message for rules validation"), [_ruleData type]];
	DLog1(DLOG_10, (@"type validation error at index %d : %@", *_errorIndex, *_errorMessage));
	return NO;
}


- (IBAction)performRulesTest:(id)sender
{
	NSString* urlString = [testUrlTextView stringValue];
	NSURL* url = [NSURL URLWithString:urlString];
	NSString* logResults = nil;
	
	PithHelmet* phTest = [[PithHelmet alloc] init];
	[phTest setAllowRules:[PithHelmetPreferences makeRulesArrayFromRuleNode:allowRulesGroup]];
	[phTest setBlockRules:[PithHelmetPreferences makeRulesArrayFromRuleNode:blockRulesGroup]];
	[phTest setExemptSiteRules:[PithHelmetPreferences makeRulesArrayFromRuleNode:exemptSiteRulesGroup]];
	
	BOOL allowed = [phTest allowResourceLoad:url withReferrer:nil testMode:YES logResults:&logResults];
	NSMutableString* resultDescription = [NSMutableString stringWithString: allowed ? NSLocalizedStringFromTableInBundle(@"overall result: allowed", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Test panel result message") : NSLocalizedStringFromTableInBundle(@"overall result: blocked", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Test panel result message")];
	
	if (logResults != nil)
	{
		[resultDescription appendString:@"\n"];
		[resultDescription appendString:logResults];
	}
	
	if (allowed == NO)
	{
		[resultDescription appendString:NSLocalizedStringFromTableInBundle(@"blocked file replacement: ", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Test panel result message")];
		[resultDescription appendString:[[[PithHelmet sharedInstance] blockedFileForUrl:[NSURL URLWithString:urlString]] absoluteString]];
	}
	
	NSString* transmogrifiedUrl = [[PithHelmet sharedInstance] transmogrify:url];
	if (transmogrifiedUrl != nil)
	{
		[resultDescription appendString:NSLocalizedStringFromTableInBundle(@"transmogrifier result url: ", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Test panel result message")];
		[resultDescription appendString:transmogrifiedUrl];
	}
	
	[testResultTextView setString:resultDescription];
	[phTest release];
}

- (IBAction)reportBug:(id)sender
{
	NSString* url = [NSString stringWithFormat:@"mailto:pithhelmet-bug@culater.net?subject=PithHelmet v%@-%@ Bug Report (Safari v%@)", [[PithHelmet sharedInstance] version], [[PithHelmet sharedInstance] bundleVersion], [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleVersion"]];
	DLog1(DLOG_10, (@"reportBug: %@", url));
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:[url urlEncode]]];
}


- (IBAction)toggleCheckbox:(id)sender
{
	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	NSString* keyName = nil;
	
	if (sender == collapseBlockedContentCheckbox)
		keyName = PH_PREF_KEY_COLLAPSE_BLOCKED_CONTENT;
	else if (sender == enableContextualMenuCheckbox)
		keyName = PH_PREF_KEY_ENABLE_CONTEXTUAL_MENU;
	else if (sender == enableLoggingCheckbox)
		keyName = PH_PREF_KEY_ENABLE_LOGGING;
	else if (sender == enableTransmogrifierCheckbox)
		keyName = PH_PREF_KEY_ENABLE_TRANSMOGRIFIER;
	else if (sender == enableDebugMenuCheckbox)
		keyName = PH_PREF_KEY_ENABLE_DEBUG_MENU;
	else if (sender == blockCrossSiteContentCheckbox)
		keyName = PH_PREF_KEY_BLOCK_CROSS_SITE_CONTENT;
	else if (sender == collapseAdSizedImagesCheckbox)
		keyName = PH_PREF_KEY_COLLAPSE_AD_SIZED_IMAGES;
	else if (sender == collapseBlockedLinksCheckbox)
		keyName = PH_PREF_KEY_COLLAPSE_BLOCKED_LINKS;
	else
		NSLog(@"trying to toggle unknown checkbox");
	
	[defaults setBool:[sender state] == NSOnState ? YES : NO forKey:keyName];

	// if we changed this, let's rebuild the damn thing, probably we should just wait for
	// people to hit save of something...
	if (sender == collapseAdSizedImagesCheckbox || sender == collapseBlockedLinksCheckbox)
		[[PithHelmet sharedInstance] buildContentScriptlet];

	if (sender == collapseBlockedContentCheckbox)
		[self initializeFromDefaults];

	DLog1(DLOG_10, (@"toggling key %@", keyName));
}


- (IBAction)setPlayAnimatedGIFs:(id)sender
{
	if ([sender isKindOfClass:[NSPopUpButton class]] != YES)
		return;

	NSString* playValue;
	NSMenuItem* menuItem = [(NSPopUpButton*)sender selectedItem];
		
	switch ([menuItem tag])
	{
		case PH_MENU_TAG_GIF_ANIMATIONS_PLAY_ONCE:
			playValue = PH_PREF_VAL_PLAY_ANIMATED_GIFS_ONCE;
			break;

		case PH_MENU_TAG_GIF_ANIMATIONS_PLAY_NEVER:
			playValue = PH_PREF_VAL_PLAY_ANIMATED_GIFS_NEVER;
			break;

		case PH_MENU_TAG_GIF_ANIMATIONS_PLAY_ALWAYS:
		default:
			playValue = PH_PREF_VAL_PLAY_ANIMATED_GIFS_ALWAYS;
			break;
	}

	[[NSUserDefaults standardUserDefaults] setObject:playValue forKey:PH_PREF_KEY_PLAY_ANIMATED_GIFS];
}


- (IBAction)toggleContentFiltering:(id)sender
{
	[sender setState: [sender state] == NSOnState ? NSOffState : NSOnState];
	[[PithHelmet sharedInstance] toggleContentFiltering];
}

- (IBAction)reloadPageUnfiltered:(id)sender;
{
	[[PithHelmet sharedInstance] reloadWithoutFilter];
}

- (void) awakeFromNib
{
	DLog1(DLOG_20, (@"awakeFromNib"));
	
	NSArray* menuItems = [safariMenuAdditions itemArray];
	NSMenu* applicationMenu = [[[[NSApplication sharedApplication] mainMenu] itemAtIndex: 0] submenu];
	NSEnumerator* menuItemsEnumerator = [menuItems objectEnumerator];
	NSMenuItem* menuItem;
	while (menuItem = [menuItemsEnumerator nextObject])
	{
		[safariMenuAdditions removeItem:menuItem];
		[applicationMenu insertItem:menuItem atIndex:5];
		[menuItem setTarget:self];
		[menuItem setEnabled:YES];
		[menuItem setState:([[NSUserDefaults standardUserDefaults] boolForKey:PH_PREF_KEY_ENABLE_FILTERING] == YES) ?  NSOnState : NSOffState];
	}
	
	menuItems = [viewMenuAdditions itemArray];
	NSMenu* viewMenu = [[[[NSApplication sharedApplication] mainMenu] itemAtIndex: 3] submenu];
	menuItemsEnumerator = [menuItems objectEnumerator];
	while (menuItem = [menuItemsEnumerator nextObject])
	{
		[viewMenuAdditions removeItem:menuItem];
		[menuItem setTarget:self];
		[menuItem setEnabled:YES];
		[viewMenu insertItem:menuItem atIndex:16];
	}

	// save the original string as the format
	//FIXME: probably should release this later - only one of these should be around but shoddy style
	statsLabelFormat = [[statsLabel stringValue] copy];
	
	// this happens once, so we'll just handle that here
	[buildLabel setStringValue:[NSString stringWithFormat:[buildLabel stringValue], [[PithHelmet sharedInstance] version], [[PithHelmet sharedInstance] bundleVersion]]];
	
	// make the rules editor somewhat pretty by changing the font size to something more pleasing
	int fontSize = [[NSUserDefaults standardUserDefaults] integerForKey:PH_PREF_KEY_RULES_OUTLINE_VIEW_FONT_SIZE];
//	[[[rulesOutlineView outlineTableColumn] dataCell] setFont:[NSFont systemFontOfSize: fontSize > 0 ? fontSize : 11]];
//	float levelIndentation = 8.0;
//	[rulesOutlineView setIndentationPerLevel:levelIndentation];

	NSTableColumn* tableColumn = [rulesOutlineView tableColumnWithIdentifier:COLUMNID_NAME];
	ImageAndTextCell* imageAndTextCell = [[[ImageAndTextCell alloc] init] autorelease];
	[imageAndTextCell setEditable:YES];
	[imageAndTextCell setFont:[NSFont systemFontOfSize: fontSize > 0 ? fontSize : 11]];
	[tableColumn setDataCell:imageAndTextCell];
	
	// fixme: #define these string constants
	[PithHelmetPreferences preloadImage:@"URL"];
	[PithHelmetPreferences preloadImage:@"World"];
	[PithHelmetPreferences preloadImage:@"BoringWorld"];
	[PithHelmetPreferences preloadImage:@"InvalidRule"];
	
	treeData = [[PithHelmetRuleNode treeNodeWithData:[PithHelmetNodeData groupDataWithName:@"root"]] retain];
	
	blockRulesGroup = [PithHelmetRuleNode treeNodeWithData:[PithHelmetNodeData groupDataWithName:NSLocalizedStringFromTableInBundle(@"Block", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Rule editor group name")]];
	[treeData appendChild:blockRulesGroup];
	allowRulesGroup = [PithHelmetRuleNode treeNodeWithData:[PithHelmetNodeData groupDataWithName:NSLocalizedStringFromTableInBundle(@"Allow", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Rule editor group name")]];
	[treeData appendChild:allowRulesGroup];
	exemptSiteRulesGroup = [PithHelmetRuleNode treeNodeWithData:[PithHelmetNodeData groupDataWithName:NSLocalizedStringFromTableInBundle(@"Exempt Sites", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Rule editor group name")]];
	[treeData appendChild:exemptSiteRulesGroup];

	[self readRulesFromPithHelmet];
}


- (void) initializeFromDefaults
{
	DLog1(DLOG_20, (@"initializeFromDefaults"));
	
	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];

	[collapseBlockedContentCheckbox setState:[defaults boolForKey:PH_PREF_KEY_COLLAPSE_BLOCKED_CONTENT] ? NSOnState : NSOffState];
	[collapseAdSizedImagesCheckbox setState:[defaults boolForKey:PH_PREF_KEY_COLLAPSE_AD_SIZED_IMAGES] ? NSOnState : NSOffState];
	[collapseBlockedLinksCheckbox setState:[defaults boolForKey:PH_PREF_KEY_COLLAPSE_BLOCKED_LINKS] ? NSOnState : NSOffState];

	[collapseAdSizedImagesCheckbox setEnabled:[defaults boolForKey:PH_PREF_KEY_COLLAPSE_BLOCKED_CONTENT]];
	[collapseBlockedLinksCheckbox setEnabled:[defaults boolForKey:PH_PREF_KEY_COLLAPSE_BLOCKED_CONTENT]];

	[enableContextualMenuCheckbox setState: [defaults boolForKey:PH_PREF_KEY_ENABLE_CONTEXTUAL_MENU] ? NSOnState : NSOffState];
	[enableLoggingCheckbox setState:[defaults boolForKey:PH_PREF_KEY_ENABLE_LOGGING] ? NSOnState : NSOffState];
	[enableTransmogrifierCheckbox setState:[defaults boolForKey:PH_PREF_KEY_ENABLE_TRANSMOGRIFIER] ? NSOnState : NSOffState];
	[enableDebugMenuCheckbox setState:[defaults boolForKey:PH_PREF_KEY_ENABLE_DEBUG_MENU] ? NSOnState : NSOffState];
	[blockCrossSiteContentCheckbox setState:[defaults boolForKey:PH_PREF_KEY_BLOCK_CROSS_SITE_CONTENT] ? NSOnState : NSOffState];

	NSString* playValue = [defaults stringForKey:PH_PREF_KEY_PLAY_ANIMATED_GIFS];
	int playMenuTag = PH_MENU_TAG_GIF_ANIMATIONS_PLAY_ALWAYS;
	if ([playValue isEqualToString:PH_PREF_VAL_PLAY_ANIMATED_GIFS_NEVER] == YES)
		playMenuTag = PH_MENU_TAG_GIF_ANIMATIONS_PLAY_NEVER;
	else if ([playValue isEqualToString:PH_PREF_VAL_PLAY_ANIMATED_GIFS_ONCE] == YES)
		playMenuTag = PH_MENU_TAG_GIF_ANIMATIONS_PLAY_ONCE;

	[playAnimatedGIFsPopUp selectItemAtIndex:[playAnimatedGIFsPopUp indexOfItemWithTag:playMenuTag]];
}

+ (NSImage*) preloadImage:(NSString*)_name
{
	NSImage* image = nil;
	NSString* imagePath = [[NSBundle bundleWithIdentifier:PH_IDENTIFIER] pathForImageResource:_name];
	if (!imagePath)
	{
		NSLog(@"imagePath for %@ is nil", _name);
		return nil;
	}
		
	image = [[NSImage alloc] initByReferencingFile:imagePath];
	if (!image)
	{
		NSLog(@"image for %@ is nil", _name);
		return nil;
	}
		
	[image setName:_name];
	return image;
}

- (NSImage *) imageForPreferenceNamed:(NSString *)aName
{
	static NSImage* image = nil;
	if (image == nil)
		image = [PithHelmetPreferences preloadImage:PH_IMAGE_NAME];

	if (!image)
		NSLog(@"returning image for PithHelmetPreferences is nil");

	return image;
	
// sadly, this one-liner doens't work - image not sure why
//	return [NSImage imageNamed:PH_IMAGE_NAME];
}

- (NSString *) preferencesNibName
{
	DLog1(DLOG_20, (@"preferencesNibName"));
	return @"PithHelmetPreferences";
}

- (void) didChange
{
	DLog1(DLOG_10, (@"didChange"));
	[super didChange];
}


- (NSView*) viewForPreferenceNamed:(NSString *)aName;
{
	if ([[PithHelmet sharedInstance] isLoaded] == NO)
		return nil;

	NSView* view = [super viewForPreferenceNamed:aName];
	return view;
}

/**
 * Called when switching preference panels.
 */
- (void) willBeDisplayed
{
	DLog1(DLOG_20, (@"willBeDisplayed"));

	// bail early if we couldn't load the bundle properly
	if ([[PithHelmet sharedInstance] isLoaded] == NO)
		return;

	[self initializeFromDefaults];

	if (statsLabelFormat == nil)
		NSLog(@"holy crap - statsLabelFormat == nil");
		
	NSString* formatted = [NSString stringWithFormat: statsLabelFormat, [[PithHelmet sharedInstance] blockedUrlCount], [[PithHelmet sharedInstance] totalUrlCount], [[PithHelmet sharedInstance] blockedPercentage]];
	[statsLabel setStringValue: formatted];
	
	// FIXME: do i really need to do this?
	[rulesOutlineView reloadData];
}

/**
 * NOTE: i tried to iterate through the checkboxes and save them to the NSUserDefaults here,
 * but the defaults object kept crashing with EXC_BAD_ACCESS.  I have no idea why.
 */
- (void) saveChanges
{
	DLog1(DLOG_20, (@"saveChanges"));

	// bail early if we couldn't load the bundle properly
	if ([[PithHelmet sharedInstance] isLoaded] == NO)
		return;

	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	[defaults synchronize];
	
	NSMutableArray* allowRules = [PithHelmetPreferences makeRulesArrayFromRuleNode:allowRulesGroup];
	NSMutableArray* blockRules = [PithHelmetPreferences makeRulesArrayFromRuleNode:blockRulesGroup];
	NSMutableArray* exemptSiteRules = [PithHelmetPreferences makeRulesArrayFromRuleNode:exemptSiteRulesGroup];
	
	[[PithHelmet sharedInstance] setAllowRules:allowRules];
	[[PithHelmet sharedInstance] setBlockRules:blockRules];
	[[PithHelmet sharedInstance] setExemptSiteRules:exemptSiteRules];
	[[PithHelmet sharedInstance] saveRulesToFile:[PH_PLIST_PATH stringByExpandingTildeInPath]];
	[[PithHelmet sharedInstance] buildContentScriptlet];
}

/**
 * Not sure how useful this is, so far always seems to return YES.
 */
- (BOOL) hasChangesPending
{
	BOOL result = [super hasChangesPending];
	DLog1(DLOG_20, (@"hasChangesPending: %d", result));
	return result;
}

/**
 * Called when we relinquish ownership of the preferences panel.
 */
- (void)moduleWillBeRemoved
{
	DLog1(DLOG_20, (@"moduleWillBeRemoved"));
	[super moduleWillBeRemoved];
}

/**
 * Called after willBeDisplayed, once we are "own" the preferences panel.
 */
- (void)moduleWasInstalled
{
	DLog1(DLOG_10, (@"moduleWasInstalled"));
	[super moduleWasInstalled];

	// just show alert if we had trouble loading
 	if ([[PithHelmet sharedInstance] isLoaded] == NO)
		[self errorAlert:NSLocalizedStringFromTableInBundle(@"An error occurred loading PithHelmet.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:NSLocalizedStringFromTableInBundle(@"The preferences pane has been disabled.", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert details")];
}



// ================================================================
//  NSOutlineView data source methods. (The required ones)
// ================================================================
#define NODE(n)			((PithHelmetRuleNode*)n)
#define NODE_DATA(n) 		((PithHelmetRuleNodeData*)[NODE((n)) nodeData])
#define SAFENODE(n) 		((PithHelmetRuleNode*)((n)?(n):(treeData)))

#define DragDropSimplePboardType 	@"MyCustomOutlineViewPboardType"


- (id)outlineView:(NSOutlineView *)_outlineView child:(int)index ofItem:(id)item
{
	PithHelmetRuleNode* node = SAFENODE(item);
	if (node == nil)
	{
		NSLog(@"nil node from item at index %d", index);
		return nil;
	}
	else if (index < 0 || index >= [node numberOfChildren])
	{
		NSLog(@"out of bounds index %d, number of children %d", index, [node numberOfChildren]);
		return nil;
	}
	
	return [SAFENODE(item) childAtIndex:index];
}

- (BOOL)outlineView:(NSOutlineView *)olv isItemExpandable:(id)item
{
	return [item isGroup];
}

- (int)outlineView:(NSOutlineView *)olv numberOfChildrenOfItem:(id)item
{
	DLog1(0, (@"count: %d item: %@", [SAFENODE(item) numberOfChildren], item));
	PithHelmetRuleNode* node = SAFENODE(item);
	if (node == nil)
	{
		NSLog(@"outlineView:numberOfChildren: safe node nil");
		return 0;
	}
	return [SAFENODE(item) numberOfChildren];
}

- (id)outlineView:(NSOutlineView *)olv objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	id objectValue = nil;
	DLog1(0, (@"col: %@ item: %@", [tableColumn identifier], item));
	
	// The return value from this method is used to configure the state of the items cell via setObjectValue:
	if([[tableColumn identifier] isEqualToString:COLUMNID_NAME])
	{
		PithHelmetRuleNodeData* data = NODE_DATA(item);
		if (data == nil)
			NSLog(@"objectValueForTableColumn data is nil");
		else
			objectValue = [data name];  // note: this is not rule-specific so we handle groups as well
	}
	
	return objectValue;
}

- (void)outlineView:(NSOutlineView *)outlineView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
	if (item == nil)
	{
		NSLog(@"setObjectValue '%@' for nil item", object);
		return;
	}

	if([[tableColumn identifier] isEqualToString:COLUMNID_NAME])
	{
		DLog1(DLOG_10, (@"outlineView setObjectValue: <%@> %@", [object class], object));
		
		if (![[NODE_DATA(item) rule] isEqualToString:(NSString*)object])
		{
			int errorIndex = -1;
			NSString* errorMessage;
			PithHelmetRuleNodeData* ruleData = NODE_DATA(item);
			
			[ruleData setRule:object];
			
			DLog1(DLOG_10, (@"validate: <%@> %@", [ruleData class], ruleData));
			if ([PithHelmetPreferences validateRule:ruleData errorIndex:&errorIndex errorMessage:&errorMessage] != YES)
			{
				NSBeep();
				[self errorAlert:NSLocalizedStringFromTableInBundle(@"PithHelmet Rule Syntax Error", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert primary message") withDetails:NSLocalizedStringFromTableInBundle(@"Error at index %d (%@).", PH_STRING_TABLE, [NSBundle bundleWithIdentifier:PH_IDENTIFIER], @"Error alert details, substitute index and errorMessage"), errorIndex, errorMessage];
			}
		}
	}
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldEditTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	if([[tableColumn identifier] isEqualToString:COLUMNID_NAME])
	{
		return ![item isGroup];
	}
	
	return NO;
}

- (BOOL)outlineView:(NSOutlineView *)olv shouldExpandItem:(id)item
{
	return [item isGroup];
}

- (void)outlineView:(NSOutlineView *)olv willDisplayCell:(NSCell *)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
{
	if (item != nil && [[tableColumn identifier] isEqualToString:COLUMNID_NAME])
	{
		PithHelmetRuleNodeData* data = NODE_DATA(item);
		if (data == nil)
			return;

		NSImage* cellIcon = nil;
		if ([item isGroup] == YES)
			cellIcon = nil;
		else if ([data isValid] == NO)
			cellIcon = [NSImage imageNamed:@"InvalidRule"];
		else if ([[data type] isEqualToString:PH_RULE_TYPE_REGEX_MATCH])
			cellIcon = [NSImage imageNamed:@"URL"];
		else if ([[data type] isEqualToString:PH_RULE_TYPE_REGEX_DOMAIN_MATCH])
			cellIcon = [NSImage imageNamed:@"World"];
		else if ([[data type] isEqualToString:PH_RULE_TYPE_DOMAIN_MATCH])
			cellIcon = [NSImage imageNamed:@"BoringWorld"];

		[(ImageAndTextCell*)cell setImage:cellIcon];

		NSFont* cellFont = [[tableColumn dataCell] font];
		float stringWidth = [cellFont widthOfString:[data name]];

		// 20.0 is a corrective fudge factor i pulled out of my ass
		stringWidth += [olv levelForItem:item] * [olv indentationPerLevel] + 20.0;
		
		// if we are in a scrollview, tack on some extra width to counteract the width of
		// the vertical scroller (if it exists)
		if ([[olv enclosingScrollView] hasVerticalScroller] == YES)
			stringWidth += [NSScroller scrollerWidthForControlSize:[[[olv enclosingScrollView] verticalScroller] controlSize]];
		if (stringWidth > [tableColumn width])
			[tableColumn setWidth:stringWidth];
	}
}

#ifdef CRAPPLE
// ================================================================
//  NSOutlineView data source methods. (dragging related)
// ================================================================

- (BOOL)outlineView:(NSOutlineView *)olv writeItems:(NSArray*)items toPasteboard:(NSPasteboard*)pboard {
	draggedNodes = items; // Don't retain since this is just holding temporaral drag information, and it is only used during a drag!  We could put this in the pboard actually.

// Provide data for our custom type, and simple NSStrings.
	[pboard declareTypes:[NSArray arrayWithObjects: DragDropSimplePboardType, NSStringPboardType, nil] owner:self];

// the actual data doesn't matter since DragDropSimplePboardType drags aren't recognized by anyone but us!.
	[pboard setData:[NSData data] forType:DragDropSimplePboardType]; 

// Put string data on the pboard... notice you candrag into TextEdit!
	[pboard setString: [draggedNodes description] forType: NSStringPboardType];

	return YES;
}

- (unsigned int)outlineView:(NSOutlineView*)olv validateDrop:(id <NSDraggingInfo>)info proposedItem:(id)item proposedChildIndex:(int)childIndex {
// This method validates whether or not the proposal is a valid one. Returns NO if the drop should not be allowed.
	SimpleTreeNode *targetNode = item;
	BOOL targetNodeIsValid = YES;

	if ([self onlyAcceptDropOnRoot]) {
		targetNode = nil;
		childIndex = NSOutlineViewDropOnItemIndex;
	} else {
		BOOL isOnDropTypeProposal = childIndex==NSOutlineViewDropOnItemIndex;

// Refuse if: dropping "on" the view itself unless we have no data in the view.
		if (targetNode==nil && childIndex==NSOutlineViewDropOnItemIndex && [treeData numberOfChildren]!=0) 
			targetNodeIsValid = NO;

		if (targetNode==nil && childIndex==NSOutlineViewDropOnItemIndex && [self allowOnDropOnLeaf]==NO)
			targetNodeIsValid = NO;

// Refuse if: we are trying to do something which is not allowed as specified by the UI check boxes.
		if ((targetNodeIsValid && isOnDropTypeProposal==NO && [self allowBetweenDrop]==NO) ||
				([NODE_DATA(targetNode) isGroup] && isOnDropTypeProposal==YES && [self allowOnDropOnGroup]==NO) ||
				([NODE_DATA(targetNode) isLeaf ] && isOnDropTypeProposal==YES && [self allowOnDropOnLeaf]==NO))
			targetNodeIsValid = NO;

// Check to make sure we don't allow a node to be inserted into one of its descendants!
		if (targetNodeIsValid && ([info draggingSource]==outlineView) && [[info draggingPasteboard] availableTypeFromArray:[NSArray arrayWithObject: DragDropSimplePboardType]] != nil) {
			NSArray *_draggedNodes = [[[info draggingSource] dataSource] draggedNodes];
			targetNodeIsValid = ![targetNode isDescendantOfNodeInArray: _draggedNodes];
		}
	}

// Set the item and child index in case we computed a retargeted one.
	[outlineView setDropItem:targetNode dropChildIndex:childIndex];

	return targetNodeIsValid ? NSDragOperationGeneric : NSDragOperationNone;
}

- (void)_performDropOperation:(id <NSDraggingInfo>)info onNode:(TreeNode*)parentNode atIndex:(int)childIndex {
// Helper method to insert dropped data into the model. 
	NSPasteboard * pboard = [info draggingPasteboard];
	NSMutableArray * itemsToSelect = nil;

// Do the appropriate thing depending on wether the data is DragDropSimplePboardType or NSStringPboardType.
	if ([pboard availableTypeFromArray:[NSArray arrayWithObjects:DragDropSimplePboardType, nil]] != nil) {
		AppController *dragDataSource = [[info draggingSource] dataSource];
		NSArray *_draggedNodes = [TreeNode minimumNodeCoverFromNodesInArray: [dragDataSource draggedNodes]];
		NSEnumerator *draggedNodesEnum = [_draggedNodes objectEnumerator];
		SimpleTreeNode *_draggedNode = nil, *_draggedNodeParent = nil;

		itemsToSelect = [NSMutableArray arrayWithArray:[self selectedNodes]];

		while ((_draggedNode = [draggedNodesEnum nextObject])) {
			_draggedNodeParent = (SimpleTreeNode*)[_draggedNode nodeParent];
			if (parentNode==_draggedNodeParent && [parentNode indexOfChild: _draggedNode]<childIndex) childIndex--;
			[_draggedNodeParent removeChild: _draggedNode];
		}
		[parentNode insertChildren: _draggedNodes atIndex: childIndex];
	} 
	else if ([pboard availableTypeFromArray:[NSArray arrayWithObject: NSStringPboardType]]) {
		NSString *string = [pboard stringForType: NSStringPboardType];
		SimpleTreeNode *newItem = [SimpleTreeNode treeNodeWithData: [SimpleNodeData leafDataWithName:string]];

		itemsToSelect = [NSMutableArray arrayWithObject: newItem];
		[parentNode insertChild: newItem atIndex:childIndex++];
	}

	[outlineView reloadData];
	[outlineView selectItems: itemsToSelect byExtendingSelection: NO];
}

- (BOOL)outlineView:(NSOutlineView*)olv acceptDrop:(id <NSDraggingInfo>)info item:(id)targetItem childIndex:(int)childIndex {
	TreeNode * 		parentNode = nil;

// Determine the parent to insert into and the child index to insert at.
	if ([NODE_DATA(targetItem) isLeaf]) {
		parentNode = (SimpleTreeNode*)(childIndex==NSOutlineViewDropOnItemIndex ? [targetItem nodeParent] : targetItem);
		childIndex = (childIndex==NSOutlineViewDropOnItemIndex ? [[targetItem nodeParent] indexOfChild: targetItem]+1 : 0);
		if ([NODE_DATA(parentNode) isLeaf]) [NODE_DATA(parentNode) setIsLeaf:NO];
	} else {            
		parentNode = SAFENODE(targetItem);
		childIndex = (childIndex==NSOutlineViewDropOnItemIndex?0:childIndex);
	}

	[self _performDropOperation:info onNode:parentNode atIndex:childIndex];

	return YES;
}
#endif

- (NSMenu*) contextMenu
{
	return contextMenu;
}

- (NSMenu*) imageContextMenuAdditions
{
	return imageContextMenuAdditions;
}

- (NSMenu*) webContextMenuAdditions
{
	return webContextMenuAdditions;
}

- (void) errorAlert:(NSString*)_message withDetails:(NSString*)_details, ...
{
	va_list ap;
	va_start(ap, _details);
	NSString* detailsFormatted = [[[NSString alloc] initWithFormat:_details arguments:ap] autorelease];
	va_end(ap);
	NSLog(@"%@\n%@", _message, detailsFormatted);
	NSWindow* window = [[NSPreferences sharedPreferences] window];
	
	if (window != nil && [window isVisible] == YES)
		NSBeginAlertSheet(
			_message,     // sheet message
			nil,            // default button label
			nil,              // alternate button label
			nil,              // no third button
			window,           // window sheet is attached to
			self,             // we'll be our own delegate
			nil,              // did-end selector
			nil,              // no need for did-dismiss selector
			nil,           // context info
			detailsFormatted,	// additional text
			nil);             // no parameters in message
}

@end
