/**
 * PithHelmet
 * @author Mike Solomon
 * Copyright 2003 Mike Solomon. All rights reserved.
 * $Header: /Users/msolomon/Developer/cvs/PithHelmet/PithHelmetRuleNode.h,v 1.9 2003/05/16 07:07:03 msolomon Exp $
 */

#import <Foundation/Foundation.h>
#import <Cocoa/Cocoa.h>
#import "TreeNode.h"
#import "PCRegExp.h"


#define PH_RULE_KEY_RULE_TYPE @"RuleType"
#define PH_RULE_KEY_FACTORY_RULE @"FactoryRule"
#define PH_RULE_KEY_SUBRESOURCE_RULE @"SubresourceRule"
#define PH_RULE_KEY_MATCH_PATTERN @"MatchPattern"
#define PH_RULE_KEY_REPLACEMENT_STRING @"ReplacementString"

#define PH_RULE_TYPE_DOMAIN_MATCH @"domain_match"
#define PH_RULE_TYPE_REGEX_MATCH @"regex_match"
#define PH_RULE_TYPE_REGEX_DOMAIN_MATCH @"regex_domain_match"
#define PH_RULE_TYPE_REGEX_REPLACE @"regex_replace"


@interface PithHelmetNodeData : TreeNodeData
{
@protected
	NSString* name;
}

+ (id)groupDataWithName:(NSString*)_name;
- (id)initWithName:(NSString*)_name;
- (void)setName:(NSString *)_name;
- (NSString*)name;
- (NSComparisonResult)compare:(TreeNodeData*)other;

@end

@interface PithHelmetRuleNodeData : PithHelmetNodeData
{
@protected
	NSString* type;
	BOOL factoryRule;
	BOOL valid;
	BOOL subresourceRule; // apply only to subresources
}

+ (PithHelmetRuleNodeData*) ruleDataWithDictionary:(NSDictionary*)_ruleDictionary;
+ (PithHelmetRuleNodeData*) ruleDataWithRule:(NSString*)_rule andType:(NSString*)_type;
- (id)initWithRule:(NSString*)_rule andType:(NSString*)_type;
- (id)initWithDictionary:(NSDictionary*)_ruleDictionary;
- (void)setType:(NSString *)_type;
- (NSString*)type;
- (void) setRule:(NSString*)_rule;
- (NSString*)rule;
- (void)setFactoryRule:(BOOL)_factoryRule;
- (BOOL)isFactoryRule;
- (void)setSubresourceRule:(BOOL)_subresourceRule;
- (BOOL)isSubresourceRule;
- (BOOL)matchURL:(NSURL*)_url;
- (BOOL)validate;
- (BOOL)isValid;
- (NSDictionary*) asDictionary;

@end

@interface PithHelmetRuleStringMatch : PithHelmetRuleNodeData

- (BOOL)matchURL:(NSURL*)_url;

@end

@interface PithHelmetRuleDomainMatch : PithHelmetRuleStringMatch
{
}

- (BOOL)matchURL:(NSURL*)_url;

@end

@interface PithHelmetRuleRegexpMatch : PithHelmetRuleStringMatch
{
@protected
	PCRegExp* matchPattern;
}

- (BOOL)matchURL:(NSURL*)_url;

@end

@interface PithHelmetRuleRegexpDomainMatch : PithHelmetRuleRegexpMatch
{
}

- (BOOL)matchURL:(NSURL*)_url;

@end

@interface PithHelmetRuleRegexpReplace : PithHelmetRuleRegexpMatch
{
@protected
	NSString* replacementString;
}
- (id)initWithRule:(NSString*)_rule andType:(NSString*)_type replacementString:(NSString*)_replacementString;
- (void)setReplacementString:(NSString *)_replacementString;
- (NSString*)replacementString;
- (NSString*)replace:(NSURL*)_url;
@end

@interface PithHelmetRuleNode : TreeNode
{
}

+ (PithHelmetRuleNode*) ruleNodeWithDictionary:(NSDictionary*)_ruleDictionary;
+ (PithHelmetRuleNode*) ruleNodeWithRule:(NSString*)_rule andType:(NSString*)_type;
+ (PithHelmetRuleNode*) groupNodeWithName:(NSString*)_name;
- (BOOL) isGroup;
@end
