/**
 * PithHelmet
 * @author Mike Solomon
 * Copyright 2003 Mike Solomon. All rights reserved.
 * $Header: /Users/msolomon/Developer/cvs/PithHelmet/PithHelmetRuleNode.m,v 1.13 2003/11/12 07:38:41 msolomon Exp $
 */

#import "TreeNode.h"
#import "PithHelmetRuleNode.h"


@implementation PithHelmetNodeData : TreeNodeData

- (id)initWithName:(NSString*)_name
{
	[self setName:_name];
	return self;
}


- (void)setName:(NSString *)_name
{
	if (!name)
	{
		name = [_name copy]; 
	}
	else if (![name isEqualToString:_name])
	{
		[name autorelease]; 
		name = [_name copy]; 
	}
}


- (NSString*)name
{ 
	return name; 
}


- (NSString*)description
{
	return name;
}


- (void)dealloc
{
	if (name)
    [name release];
	name = nil;
		
	[super dealloc];
}


- (NSComparisonResult)compare:(TreeNodeData*)other
{
	// We want the data to be sorted by name, so we compare [self name] to [other name]
	PithHelmetNodeData* _other = (PithHelmetNodeData *)other;
	return [name compare:[_other name]];
}


+ (id)groupDataWithName:(NSString*)_name
{
	return [[[PithHelmetNodeData alloc] initWithName:_name] autorelease];
}

@end




@implementation PithHelmetRuleNodeData : PithHelmetNodeData

+ (PithHelmetRuleNodeData*) ruleDataWithDictionary:(NSDictionary*)_ruleDictionary
{
	// fixme: implement error checking in case no type is specified
	NSString* dType = [_ruleDictionary objectForKey:PH_RULE_KEY_RULE_TYPE];
	
	if ([dType isEqualToString:PH_RULE_TYPE_DOMAIN_MATCH] == YES)
		return [[[PithHelmetRuleDomainMatch alloc] initWithDictionary:_ruleDictionary] autorelease];
	else if ([dType isEqualToString:PH_RULE_TYPE_REGEX_MATCH] == YES)
		return [[[PithHelmetRuleRegexpMatch alloc] initWithDictionary:_ruleDictionary] autorelease];
	else if ([dType isEqualToString:PH_RULE_TYPE_REGEX_DOMAIN_MATCH] == YES)
		return [[[PithHelmetRuleRegexpDomainMatch alloc] initWithDictionary:_ruleDictionary] autorelease];
	else if ([dType isEqualToString:PH_RULE_TYPE_REGEX_REPLACE] == YES)
		return [[[PithHelmetRuleRegexpReplace alloc] initWithDictionary:_ruleDictionary] autorelease];

	return nil;
}


+ (PithHelmetRuleNodeData*) ruleDataWithRule:(NSString*)_rule andType:(NSString*)_type
{
	if ([_type isEqualToString:PH_RULE_TYPE_DOMAIN_MATCH] == YES)
		return [[[PithHelmetRuleDomainMatch alloc] initWithRule:_rule andType:_type] autorelease];
	else if ([_type isEqualToString:PH_RULE_TYPE_REGEX_MATCH] == YES)
		return [[[PithHelmetRuleRegexpMatch alloc] initWithRule:_rule andType:_type] autorelease];
	else if ([_type isEqualToString:PH_RULE_TYPE_REGEX_DOMAIN_MATCH] == YES)
		return [[[PithHelmetRuleRegexpDomainMatch alloc] initWithRule:_rule andType:_type] autorelease];
	else if ([_type isEqualToString:PH_RULE_TYPE_REGEX_REPLACE] == YES)
		return [[[PithHelmetRuleRegexpReplace alloc] initWithRule:_rule andType:_type] autorelease];

	return nil;	
}


- (id)initWithRule:(NSString*)_rule andType:(NSString*)_type
{
	self = [super initWithName:_rule];
	if (self == nil)
		return nil;
	
	[self setType:_type];
	[self validate];
	factoryRule = NO;
	subresourceRule = NO;
	return self;
}


- (id)initWithDictionary:(NSDictionary*)_ruleDictionary
{
	// fixme: add error checking via exceptions
	NSString* dName = [_ruleDictionary objectForKey:PH_RULE_KEY_MATCH_PATTERN];
	NSString* dType = [_ruleDictionary objectForKey:PH_RULE_KEY_RULE_TYPE];
	self = [self initWithRule:dName andType:dType];
	if (self == nil)
		return nil;
		
	BOOL dFactoryRule = [_ruleDictionary boolForKey:PH_RULE_KEY_FACTORY_RULE];
	BOOL dSubresourceRule = [_ruleDictionary boolForKey:PH_RULE_KEY_SUBRESOURCE_RULE];
	[self setFactoryRule:dFactoryRule];
	[self setSubresourceRule:dSubresourceRule];
	[self validate];
	return self;
}


- (void)setType:(NSString *)_type
{
	if (!type)
	{
		type = [_type copy]; 
	}
	else if (![type isEqualToString: _type])
	{
		[type autorelease]; 
		type = [_type copy]; 
	}
}


- (NSString*)type
{ 
	return type;
}

- (void) setRule:(NSString*)_rule
{
	[self setName:_rule];
	[self validate];
}

- (NSString*)rule
{
	return name;
}

- (void)setFactoryRule:(BOOL)_factoryRule
{
	factoryRule = _factoryRule;
}


- (BOOL)isFactoryRule
{ 
	return factoryRule;
}

- (void)setSubresourceRule:(BOOL)_subresourceRule
{
	subresourceRule = _subresourceRule;
}

- (BOOL)isSubresourceRule
{
	return subresourceRule;
}

- (BOOL)matchURL:(NSURL*)_url
{
	// fixme: this is actually an error - this should be an abstract base class
	return NO;
}

- (BOOL)isValid
{
	return valid;
}

- (BOOL)validate
{
	valid = ([[self rule] length] > 0) ? YES : NO;
	return valid;
}

/*
- (NSString*)description
{
	return [NSString stringWithFormat:@"%@ : factory(%d) : valid(%d)", name, [self isFactoryRule], [self isValid]];
}
*/

- (NSDictionary*) asDictionary
{
	NSMutableDictionary* dict  = [NSMutableDictionary dictionary];
	[dict setObject:[self rule] forKey:PH_RULE_KEY_MATCH_PATTERN];
	[dict setObject:[self type] forKey:PH_RULE_KEY_RULE_TYPE];
	[dict setObject:[NSNumber numberWithBool:[self isFactoryRule]] forKey:PH_RULE_KEY_FACTORY_RULE];
	[dict setObject:[NSNumber numberWithBool:[self isSubresourceRule]] forKey:PH_RULE_KEY_SUBRESOURCE_RULE];
	return dict;
}

- (void)dealloc
{
	if (type)
    [type release];
	type = nil;
		
	[super dealloc];
}

@end



@implementation PithHelmetRuleStringMatch : PithHelmetRuleNodeData

- (BOOL)matchURL:(NSURL*)_url
{
	// todo: implement
	// NSString* domain = [_url host];
	return NO;
}

@end


@implementation PithHelmetRuleDomainMatch : PithHelmetRuleStringMatch

- (BOOL)matchURL:(NSURL*)_url
{
	NSString* domain = [_url host];
	return (domain == nil) ? NO : [domain hasSuffix:[self rule]];
}

@end

@implementation PithHelmetRuleRegexpMatch : PithHelmetRuleStringMatch

- (id)initWithRule:(NSString*)_rule andType:(NSString*)_type
{
	self = [super initWithRule:_rule andType:_type];
	if (self == nil)
		return nil;
	
	matchPattern = [[PCRegExp alloc] initWithString:name];
	[self validate];
	return self;
}

- (void) setRule:(NSString*)_rule
{
	if (matchPattern != nil)
		[matchPattern autorelease];

	matchPattern = [[PCRegExp alloc] initWithString:name];
	// changing order since this will automatically call validate - this might be
	// a sign of bad design
	[super setRule:_rule];
}

- (BOOL)matchURL:(NSURL*)_url
{
	if ([self isValid] == NO)
		return NO;
		
	NSString* url = [_url absoluteString];
	return [matchPattern matchInString:url];
}

- (BOOL)validate
{
	valid = YES;
	NSString* ruleString = [self rule];
	int length = [ruleString length];
	if (length <= 0)
	{
		valid = NO;
		return valid;
	}

	int i;
	for (i = length - 1; i >= 0; i--)
	{
		// check for unescaped / characters - the so-called "leaning toothpick"
		// this is here because of the javascript syntax
		if ([ruleString characterAtIndex:i] == '/' && (!(i - 1 >= 0) || [ruleString characterAtIndex:i-1] != '\\'))
		{
			DLog1(DLOG_10, (@"toothpick validation error at index %d", i));
			valid = NO;
			return valid;
		}
	}

	valid = (matchPattern != nil) ? YES : NO;
	return valid;
}

- (void)dealloc
{
	if (matchPattern)
    [matchPattern release];
	matchPattern = nil;
		
	[super dealloc];
}

@end

@implementation PithHelmetRuleRegexpDomainMatch : PithHelmetRuleRegexpMatch

- (BOOL)matchURL:(NSURL*)_url
{
	if ([self isValid] == NO)
		return NO;
		
	NSString* domain = [_url host];
	return (domain == nil) ? NO : [matchPattern matchInString:domain];
}

@end


@implementation PithHelmetRuleRegexpReplace : PithHelmetRuleRegexpMatch

- (id)initWithRule:(NSString*)_rule andType:(NSString*)_type replacementString:(NSString*)_replacementString
{
	self = [super initWithRule:_rule andType:_type];
	if (self == nil)
		return nil;
	
	[self setReplacementString:_replacementString];
	[self validate];
	return self;
}

- (id) initWithDictionary:(NSDictionary*)_ruleDictionary
{
	self = [super initWithDictionary:_ruleDictionary];
	if (self == nil)
		return nil;
	
	// fixme: error checking
	NSString* dReplacementString = [_ruleDictionary objectForKey:PH_RULE_KEY_REPLACEMENT_STRING];
	[self setReplacementString:dReplacementString];
	[self validate];
	return self;
}

- (void)setReplacementString:(NSString *)_replacementString
{
	if (!replacementString)
	{
		replacementString = [_replacementString copy]; 
	}
	else if (![replacementString isEqualToString:_replacementString])
	{
		[replacementString autorelease];
		replacementString = [_replacementString copy]; 
	}
}

- (NSString*)replacementString
{ 
	return replacementString; 
}

- (BOOL)validate
{
	valid = ([super validate] == YES && replacementString != nil && [replacementString length] > 0) ? YES : NO;
	return valid;
}


- (NSString*)description
{
	return [NSString stringWithFormat:@"%@ --> %@", name, replacementString];
}


- (NSString*)replace:(NSURL*)_url
{
	if ([self matchURL:_url] == YES)
	{
		NSString* url = [_url absoluteString];
		NSString* newUrl = [matchPattern replaceString:url byPattern:replacementString];
		return newUrl;
	}
	
	return nil;
}

- (NSDictionary*) asDictionary
{
	NSMutableDictionary* dict  = [NSMutableDictionary dictionaryWithDictionary:[super asDictionary]];
	[dict setObject:[self replacementString] forKey:PH_RULE_KEY_REPLACEMENT_STRING];
	return dict;
}


- (void)dealloc
{
	if (replacementString)
    [replacementString release];
	replacementString = nil;
		
	[super dealloc];
}

@end



@implementation PithHelmetRuleNode

+ (id)treeNodeWithData:(TreeNodeData*)data
{
	return [[[PithHelmetRuleNode alloc] initWithData:data parent:nil children:[NSArray array]] autorelease]; 
}

+ (PithHelmetRuleNode*) ruleNodeWithDictionary:(NSDictionary*)_ruleDictionary
{
	return [PithHelmetRuleNode treeNodeWithData:[PithHelmetRuleNodeData ruleDataWithDictionary:_ruleDictionary]];
}

+ (PithHelmetRuleNode*) ruleNodeWithRule:(NSString*)_rule andType:(NSString*)_type
{
	return [PithHelmetRuleNode treeNodeWithData:[PithHelmetRuleNodeData ruleDataWithRule:_rule andType:_type]];
}

+ (PithHelmetRuleNode*) groupNodeWithName:(NSString*)_name
{
	return [PithHelmetRuleNode treeNodeWithData:[PithHelmetRuleNodeData groupDataWithName:_name]];
}


- (BOOL) isGroup
{
	//	BOOL isKind =  [[self nodeData] isKindOfClass:[PithHelmetNodeData class]];
	BOOL isMember = [[self nodeData] isMemberOfClass:[PithHelmetNodeData class]];
	//	NSLog(@"isGroup: <%@> <%@> isMemberOfClass: %d isKindOfClass: %d", [[self nodeData] class], [PithHelmetNodeData class], isMember, isKind);
	return isMember;
}


@end
